@@ -19,25 +19,24 @@
 """
 Methods dealing with rationals.
 """
+# isort: STDLIB
 import copy
 import itertools
-
 from fractions import Fraction
 
-from ._constants import RoundingMethods
 from ._config import BasesConfig
+from ._constants import RoundingMethods
 from ._display import String
 from ._division import NatDivision
-from ._errors import BasesAssertError
-from ._errors import BasesInvalidOperationError
-from ._errors import BasesValueError
+from ._errors import BasesAssertError, BasesInvalidOperationError, BasesValueError
 from ._nats import Nats
 
 
-class Radices():
+class Radices:
     """
     Methods for Radices.
     """
+
     # pylint: disable=too-few-public-methods
 
     @staticmethod
@@ -53,18 +52,13 @@ class Radices():
             return RoundingMethods.ROUND_HALF_DOWN
         if method is RoundingMethods.ROUND_HALF_DOWN:
             return RoundingMethods.ROUND_HALF_UP
-        if method in \
-           (RoundingMethods.ROUND_TO_ZERO, RoundingMethods.ROUND_HALF_ZERO):
+        if method in (RoundingMethods.ROUND_TO_ZERO, RoundingMethods.ROUND_HALF_ZERO):
             return method
-        raise BasesAssertError('unknown method') # pragma: no cover
+        raise BasesAssertError("unknown method")  # pragma: no cover
 
     @classmethod
     def from_rational(
-       cls,
-       value,
-       to_base,
-       precision=None,
-       method=RoundingMethods.ROUND_DOWN
+        cls, value, to_base, precision=None, method=RoundingMethods.ROUND_DOWN
     ):
         """
         Convert rational value to a base.
@@ -106,24 +100,16 @@ class Radices():
         numerator = Nats.convert_from_int(value.numerator, to_base)
         denominator = Nats.convert_from_int(value.denominator, to_base)
 
-        (integer_part, non_repeating_part, repeating_part, relation) = \
-           NatDivision.division(
-              denominator,
-              numerator,
-              to_base,
-              precision,
-              div_method
-           )
+        (
+            integer_part,
+            non_repeating_part,
+            repeating_part,
+            relation,
+        ) = NatDivision.division(denominator, numerator, to_base, precision, div_method)
 
         relation = relation * sign
 
-        result = Radix(
-           sign,
-           integer_part,
-           non_repeating_part,
-           repeating_part,
-           to_base
-        )
+        result = Radix(sign, integer_part, non_repeating_part, repeating_part, to_base)
 
         if precision is not None:
             (result, rel) = result.rounded(precision, method)
@@ -132,10 +118,11 @@ class Radices():
         return (result, relation)
 
 
-class Rationals():
+class Rationals:
     """
     Miscellaneous methods for rationals.
     """
+
     # pylint: disable=too-few-public-methods
 
     @staticmethod
@@ -186,32 +173,30 @@ class Rationals():
         raise BasesValueError(method, "method")
 
 
-class Radix():
+class Radix:
     """
     An object containing information about a rational representation.
 
     Such values can not be ordered, but can be compared for equality.
     """
+
     # pylint: disable=too-few-public-methods
 
-    _FMT_STR = "".join([
-       "%(sign)s",
-       "%(left)s",
-       "%(radix)s",
-       "%(non_repeat)s",
-       "%(repeat)s",
-       "_",
-       "%(base)s"
-    ])
+    _FMT_STR = "".join(
+        [
+            "%(sign)s",
+            "%(left)s",
+            "%(radix)s",
+            "%(non_repeat)s",
+            "%(repeat)s",
+            "_",
+            "%(base)s",
+        ]
+    )
 
     @classmethod
-    def _validate( # pylint: disable=too-many-arguments
-        cls,
-        sign,
-        integer_part,
-        non_repeating_part,
-        repeating_part,
-        base
+    def _validate(  # pylint: disable=too-many-arguments
+        cls, sign, integer_part, non_repeating_part, repeating_part, base
     ):
         """
         Check if radix is valid.
@@ -232,31 +217,25 @@ class Radix():
         """
         if any(x < 0 or x >= base for x in integer_part):
             return BasesValueError(
-               integer_part,
-               "integer_part",
-               "values must be between 0 and %s" % base
+                integer_part, "integer_part", "values must be between 0 and %s" % base
             )
         if any(x < 0 or x >= base for x in non_repeating_part):
             return BasesValueError(
-               non_repeating_part,
-               "non_repeating_part",
-               "values must be between 0 and %s" % base
+                non_repeating_part,
+                "non_repeating_part",
+                "values must be between 0 and %s" % base,
             )
         if any(x < 0 or x >= base for x in repeating_part):
             return BasesValueError(
-               repeating_part,
-               "repeating_part",
-               "values must be between 0 and %s" % base
+                repeating_part,
+                "repeating_part",
+                "values must be between 0 and %s" % base,
             )
         if base < 2:
             return BasesValueError(base, "base", "must be at least 2")
 
         if sign not in (-1, 0, 1) or sign is True or sign is False:
-            return BasesValueError(
-               sign,
-               "sign",
-               "must be an int between -1 and 1"
-            )
+            return BasesValueError(sign, "sign", "must be an int between -1 and 1")
 
         return None
 
@@ -285,8 +264,10 @@ class Radix():
             (quot, rem) = divmod(repeat_len, index)
             if rem == 0:
                 first_chunk = part[0:index]
-                if all(first_chunk == part[x:x + index] \
-                   for x in range(index, quot * index, index)):
+                if all(
+                    first_chunk == part[x : x + index]
+                    for x in range(index, quot * index, index)
+                ):
                     return index
         return repeat_len
 
@@ -315,24 +296,19 @@ class Radix():
         # * for [1, 2, 1, 2], [1,2] end is 0
         # * for [6, 2, 1, 2], [1,2] end is 2
         indices = range(len(non_repeating), -1, -repeat_len)
-        end = next( # pragma: no cover
-           i for i in indices if non_repeating[(i - repeat_len):i] != repeating
+        end = next(  # pragma: no cover
+            i for i in indices if non_repeating[(i - repeat_len) : i] != repeating
         )
 
         # for remaining, find partial match and shift repeating
         # * for [6, 2, 1, 2], [1, 2] initial end is 2, result is [6], [2, 1]
         indices = range(min(repeat_len - 1, end), 0, -1)
         index = next(
-           (i for i in indices \
-              if repeating[-i:] == non_repeating[(end-i):end]),
-           0
-        )
-        return (
-           non_repeating[:(end - index)],
-           repeating[-index:] + repeating[:-index]
+            (i for i in indices if repeating[-i:] == non_repeating[(end - i) : end]), 0
         )
+        return (non_repeating[: (end - index)], repeating[-index:] + repeating[:-index])
 
-    def __init__( # pylint: disable=too-many-arguments
+    def __init__(  # pylint: disable=too-many-arguments
         self,
         sign,
         integer_part,
@@ -340,7 +316,7 @@ class Radix():
         repeating_part,
         base,
         validate=True,
-        canonicalize=True
+        canonicalize=True,
     ):
         """
         Initializer.
@@ -363,38 +339,37 @@ class Radix():
         """
         if validate:
             error = self._validate(
-               sign,
-               integer_part,
-               non_repeating_part,
-               repeating_part,
-               base
+                sign, integer_part, non_repeating_part, repeating_part, base
             )
             if error is not None:
-                raise error # pylint: disable=raising-bad-type
+                raise error  # pylint: disable=raising-bad-type
 
         if canonicalize:
             if all(x == 0 for x in integer_part):
                 integer_part = []
 
-            repeating_part = \
-               repeating_part[0:self._repeat_length(repeating_part)]
-            (non_repeating_part, repeating_part) = \
-                self._canonicalize_fraction(non_repeating_part, repeating_part)
+            repeating_part = repeating_part[0 : self._repeat_length(repeating_part)]
+            (non_repeating_part, repeating_part) = self._canonicalize_fraction(
+                non_repeating_part, repeating_part
+            )
             if all(x == 0 for x in repeating_part):
                 repeating_part = []
 
             if repeating_part == [base - 1]:
                 repeating_part = []
-                (carry_out, non_repeating_part) = \
-                   Nats.carry_in(non_repeating_part, 1, base)
+                (carry_out, non_repeating_part) = Nats.carry_in(
+                    non_repeating_part, 1, base
+                )
                 if carry_out != 0:
-                    (carry_out, integer_part) = \
-                       Nats.carry_in(integer_part, 1, base)
+                    (carry_out, integer_part) = Nats.carry_in(integer_part, 1, base)
                     if carry_out != 0:
                         integer_part = [carry_out] + integer_part
 
-            if integer_part == [] and repeating_part == [] and \
-               all(x == 0 for x in non_repeating_part):
+            if (
+                integer_part == []
+                and repeating_part == []
+                and all(x == 0 for x in non_repeating_part)
+            ):
                 sign = 0
 
         self.sign = sign
@@ -416,32 +391,35 @@ class Radix():
         return self.getString(BasesConfig.DISPLAY_CONFIG, 0)
 
     def __repr__(self):
-        return 'Radix(%s,%s,%s,%s,%s)' % \
-           (
-              self.sign,
-              self.integer_part,
-              self.non_repeating_part,
-              self.repeating_part,
-              self.base
-           )
+        return "Radix(%s,%s,%s,%s,%s)" % (
+            self.sign,
+            self.integer_part,
+            self.non_repeating_part,
+            self.repeating_part,
+            self.base,
+        )
 
     def __eq__(self, other):
         if not isinstance(other, Radix):
             raise BasesInvalidOperationError("!=", other)
-        return self.sign == other.sign and \
-           self.integer_part == other.integer_part and \
-           self.non_repeating_part == other.non_repeating_part and \
-           self.repeating_part == other.repeating_part and \
-           self.base == other.base
+        return (
+            self.sign == other.sign
+            and self.integer_part == other.integer_part
+            and self.non_repeating_part == other.non_repeating_part
+            and self.repeating_part == other.repeating_part
+            and self.base == other.base
+        )
 
     def __ne__(self, other):
         if not isinstance(other, Radix):
             raise BasesInvalidOperationError("!=", other)
-        return self.sign != other.sign or \
-           self.integer_part != other.integer_part or \
-           self.non_repeating_part != other.non_repeating_part or \
-           self.repeating_part != other.repeating_part or \
-           self.base != other.base
+        return (
+            self.sign != other.sign
+            or self.integer_part != other.integer_part
+            or self.non_repeating_part != other.non_repeating_part
+            or self.repeating_part != other.repeating_part
+            or self.base != other.base
+        )
 
     def __lt__(self, other):
         raise BasesInvalidOperationError("<")
@@ -455,22 +433,22 @@ class Radix():
     def __ge__(self, other):
         raise BasesInvalidOperationError(">=")
 
-    def __copy__(self): # pragma: no cover
+    def __copy__(self):  # pragma: no cover
         return Radix(
-           self.sign,
-           self.integer_part,
-           self.non_repeating_part,
-           self.repeating_part,
-           self.base
+            self.sign,
+            self.integer_part,
+            self.non_repeating_part,
+            self.repeating_part,
+            self.base,
         )
 
     def __deepcopy__(self, memo):
         return Radix(
-           self.sign,
-           self.integer_part[:],
-           self.non_repeating_part[:],
-           self.repeating_part[:],
-           self.base
+            self.sign,
+            self.integer_part[:],
+            self.non_repeating_part[:],
+            self.repeating_part[:],
+            self.base,
         )
 
     def as_rational(self):
@@ -480,16 +458,12 @@ class Radix():
         :returns: this radix as a rational
         :rtype: Rational
         """
-        (denominator, numerator) = \
-           NatDivision.undivision(
-              self.integer_part,
-              self.non_repeating_part,
-              self.repeating_part,
-              self.base
-           )
+        (denominator, numerator) = NatDivision.undivision(
+            self.integer_part, self.non_repeating_part, self.repeating_part, self.base
+        )
         result = Fraction(
-           Nats.convert_to_int(numerator, self.base),
-           Nats.convert_to_int(denominator, self.base)
+            Nats.convert_to_int(numerator, self.base),
+            Nats.convert_to_int(denominator, self.base),
         )
         return result * self.sign
 
@@ -536,12 +510,12 @@ class Radix():
         return result
 
 
-class _Rounding():
+class _Rounding:
     """
     Rounding of radix objects.
     """
-    # pylint: disable=too-few-public-methods
 
+    # pylint: disable=too-few-public-methods
 
     @staticmethod
     def _conditional_toward_zero(method, sign):
@@ -554,9 +528,11 @@ class _Rounding():
 
         Complexity: O(1)
         """
-        return method is RoundingMethods.ROUND_HALF_ZERO or \
-           (method is RoundingMethods.ROUND_HALF_DOWN and sign == 1) or \
-           (method is RoundingMethods.ROUND_HALF_UP and sign == -1)
+        return (
+            method is RoundingMethods.ROUND_HALF_ZERO
+            or (method is RoundingMethods.ROUND_HALF_DOWN and sign == 1)
+            or (method is RoundingMethods.ROUND_HALF_UP and sign == -1)
+        )
 
     @staticmethod
     def _increment(sign, integer_part, non_repeating_part, base):
@@ -575,17 +551,15 @@ class _Rounding():
 
         Complexity: O(len(non_repeating_part + integer_part)
         """
-        (carry, non_repeating_part) = \
-           Nats.carry_in(non_repeating_part, 1, base)
-        (carry, integer_part) = \
-           Nats.carry_in(integer_part, carry, base)
+        (carry, non_repeating_part) = Nats.carry_in(non_repeating_part, 1, base)
+        (carry, integer_part) = Nats.carry_in(integer_part, carry, base)
         return Radix(
-           sign,
-           integer_part if carry == 0 else [carry] + integer_part,
-           non_repeating_part,
-           [],
-           base,
-           False
+            sign,
+            integer_part if carry == 0 else [carry] + integer_part,
+            non_repeating_part,
+            [],
+            base,
+            False,
         )
 
     @classmethod
@@ -606,47 +580,34 @@ class _Rounding():
         # pylint: disable=too-many-branches
 
         if precision < 0:
-            raise BasesValueError(
-               precision,
-               "precision",
-               "must be at least 0"
-            )
+            raise BasesValueError(precision, "precision", "must be at least 0")
 
         if method not in RoundingMethods.METHODS():
             raise BasesValueError(
-               method,
-               "method",
-               "must be one of RoundingMethod.METHODS()"
+                method, "method", "must be one of RoundingMethod.METHODS()"
             )
 
         if value.sign == 0:
             return (Radix(0, [], precision * [0], [], value.base), 0)
 
         digits = itertools.chain(
-           value.non_repeating_part,
-           itertools.cycle(value.repeating_part)
+            value.non_repeating_part, itertools.cycle(value.repeating_part)
         )
         non_repeating_part = list(itertools.islice(digits, 0, precision))
         non_repeating_part += (precision - len(non_repeating_part)) * [0]
 
         truncated = lambda: Radix(
-           value.sign,
-           value.integer_part,
-           non_repeating_part,
-           [],
-           value.base,
-           False
+            value.sign, value.integer_part, non_repeating_part, [], value.base, False
         )
 
         incremented = lambda: cls._increment(
-           value.sign,
-           value.integer_part,
-           non_repeating_part,
-           value.base
+            value.sign, value.integer_part, non_repeating_part, value.base
         )
 
-        if all(x == 0 for x in value.non_repeating_part[precision:]) and \
-           value.repeating_part == []:
+        if (
+            all(x == 0 for x in value.non_repeating_part[precision:])
+            and value.repeating_part == []
+        ):
             return (truncated(), 0)
 
         if method is RoundingMethods.ROUND_TO_ZERO:
@@ -660,18 +621,11 @@ class _Rounding():
 
         non_repeating_remainder = value.non_repeating_part[precision:]
         if non_repeating_remainder == []:
-            repeating_part = \
-               list(itertools.islice(digits, len(value.repeating_part)))
+            repeating_part = list(itertools.islice(digits, len(value.repeating_part)))
         else:
             repeating_part = value.repeating_part[:]
 
-        remainder = Radix(
-           1,
-           [],
-           non_repeating_remainder,
-           repeating_part,
-           value.base
-        )
+        remainder = Radix(1, [], non_repeating_remainder, repeating_part, value.base)
         remainder_fraction = remainder.as_rational()
         middle = Fraction(1, 2)
         if remainder_fraction < middle:
